__all__ = ["DEFAULT_ACCEPTED_FILE_FORMATS", "DEFAULT_ACCEPTED_IMAGE_FORMATS"]

DEFAULT_ACCEPTED_IMAGE_FORMATS = (
    "png",
    "PNG",
    "jpg",
    "JPG",
    "jpeg",
    "JPEG",
    "gif",
    "GIF",
)

DEFAULT_ACCEPTED_FILE_FORMATS = DEFAULT_ACCEPTED_IMAGE_FORMATS
