#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"

from .atari_weight_init import *
from .fan_in_weight_init import *

from .normal_weight_init import *
from .ortho_weight_init import *
