#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = r"""

           Created on 9/14/19
           """

from .terminal_plot import *
from .terminal_plot_writer import *
from .console_writer import *
