#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = r"""

           Created on 01-09-2020
           """

import torch


def generate_tag(model: torch.nn.Module, **satellite_parameters) -> str:
    return ""
